/*
**  ComposeViewController.h
**
**  Copyright (c) 2001-2003 Ludovic Marcotte
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _GNUMail_H_ComposeViewController
#define _GNUMail_H_ComposeViewController

#import <AppKit/AppKit.h>

#include "PreferencesModule.h"

@interface ComposeViewController : NSObject <PreferencesModule>
{
  // Outlets
  IBOutlet id view;
  
  IBOutlet NSPopUpButton *replyPopUpButton;
  IBOutlet NSPopUpButton *forwardPopUpButton;
 
  IBOutlet NSTextField *lineWrapLimitField;

  IBOutlet NSPopUpButton *defaultCharsetPopUpButton;

  // Other ivars
}

@end


//
// Private interface
//
@interface ComposeViewController (Private)

- (void) _initializeCharsetPopUpButton;

@end

#endif // _GNUMail_H_ComposeViewController
